package jp.co.argo21.nautica.workflow.soap;

import javax.annotation.Resource;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import org.apache.log4j.Logger;

import jp.co.argo21.nautica.workflow.client.WorkflowServiceAccessor;
import jp.co.argo21.nautica.workflow.soap.message.ActivityState;
import jp.co.argo21.nautica.workflow.soap.message.Attribute;
import jp.co.argo21.nautica.workflow.soap.message.Filter;
import jp.co.argo21.nautica.workflow.soap.message.ProcessState;
import jp.co.argo21.nautica.workflow.util.StringManager;
import jp.co.argo21.nautica.workflow.wfmc.InvalidActivityNameException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidAttributeException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidFilterException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessDefinitionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidProcessInstanceException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidSessionException;
import jp.co.argo21.nautica.workflow.wfmc.InvalidStateException;
import jp.co.argo21.nautica.workflow.wfmc.TransitionNotAllowedException;

@WebService(serviceName = "WorkflowAdminHandler",
        portName="WorkflowAdminHandlerPort",
        endpointInterface = "jp.co.argo21.nautica.workflow.soap.WorkflowAdminHandler",
        targetNamespace = "http://www.argo21.co.jp/nautica/workflow/soap",
        wsdlLocation = "WEB-INF/wsdl/WorkflowAdminHandler.wsdl")
public class WorkflowAdminHandlerSoapBindingImpl implements
		WorkflowAdminHandler {
	
	/** ログ出力用オブジェクト */
	private static Logger log = Logger.getLogger(
			WorkflowAdminHandlerSoapBindingImpl.class);
	
	@Resource
    private WebServiceContext context;

	/**
	 * 指定されたプロセスインスタンスの中断を行う。
	 *
	 * @param session セッションID
	 * @param pid プロセスインスタンスID
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessInstanceException プロセスインスタンスIDが有効でない場合
	 */
	@Override
	public void abortProcess(String session, String pid) throws InvalidSessionException, InvalidProcessInstanceException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessInstanceID=" + pid + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.abortProcess(session, pid);
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessInstanceException e) {
			String errMsg = StringManager.get("E0226");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、中断を行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public void abortProcesses(String session, String pdid, Filter[] filter) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.abortProcesses(
					session, 
					pdid, 
					SoapMessageConverter.convertToGroupFilter(filter));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義とアクティビティ定義に属する
	 * アクティビティインスタンスをフィルターで絞り込み、
	 * 属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param adid アクティビティ定義ID
	 * @param filter 検索用フィルター
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidActivityNameException 指定されたアクティビティ定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@Override
	public void assignActivitiesAttribute(String session, String pdid,
			String adid, Filter[] filter, Attribute attr) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidActivityNameException, InvalidFilterException, InvalidAttributeException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", ActivityDefinitionID=" + adid 
			+ ", Filter=" + filter  
			+ ", Attribute=" + attr + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.assignActivitiesAttribute(
					session, 
					pdid, 
					adid, 
					SoapMessageConverter.convertToGroupFilter(filter), 
					attr.toWfmcAttribute());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidActivityNameException e) {
			String errMsg = StringManager.get("E0237");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidAttributeException e) {
			String errMsg = StringManager.get("E0233");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、属性の割り当てを行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @param attr 属性
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidAttributeException 属性が有効でない場合
	 */
	@Override
	public void assignProcessesAttribute(String session, String pdid,
			Filter[] filter, Attribute attr) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException, InvalidAttributeException {
		
		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", Filter" + filter 
			+ ", Attribute=" + attr + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.assignProcessesAttribute(
					session, 
					pdid, 
					SoapMessageConverter.convertToGroupFilter(filter), 
					attr.toWfmcAttribute());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidAttributeException e) {
			String errMsg = StringManager.get("E0233");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義とアクティビティ定義に属する
	 * アクティビティインスタンスをフィルターで絞り込み、状態の変更を行う。
	 * 指定可能なフィルターはStateFilterとする。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param adid アクティビティ定義ID
	 * @param filter 検索用フィルター
	 * @param state 新しいアクティビティインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidActivityNameException 指定されたアクティビティ定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@Override
	public void changeActivitiesState(String session, String pdid, String adid,
			Filter[] filter, ActivityState state) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidActivityNameException, InvalidFilterException, InvalidStateException, TransitionNotAllowedException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", ActivityDefinitionID=" + adid
			+ ", Filter" + filter  
			+ ", ActivityInstanceState=" + state + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.changeActivitiesState(
					session, 
					pdid, 
					adid, 
					SoapMessageConverter.convertToStateFilter(filter), 
					state.toWfmcActivityState());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidActivityNameException e) {
			String errMsg = StringManager.get("E0237");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidStateException e) {
			String errMsg = StringManager.get("E0234");
			log.error(errMsg, e);
			throw e;
		} catch (TransitionNotAllowedException e) {
			String errMsg = StringManager.get("E0050");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、状態の変更を行う。
	 * 指定可能なフィルターはStateFilterとする。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @param state 新しいプロセスインスタンス状態
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 * @throws InvalidStateException 指定された状態が有効でない場合
	 * @throws TransitionNotAllowedException 状態の変更が許可されていない場合
	 */
	@Override
	public void changeProcessesState(String session, String pdid,
			Filter[] filter, ProcessState state) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException, InvalidStateException, TransitionNotAllowedException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", Filter=" + filter  
			+ ", ProcessInstanceState=" + state + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.changeProcessesState(
					session, 
					pdid, 
					SoapMessageConverter.convertToStateFilter(filter), 
					state.toWfmcProcessState());
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidStateException e) {
			String errMsg = StringManager.get("E0234");
			log.error(errMsg, e);
			throw e;
		} catch (TransitionNotAllowedException e) {
			String errMsg = StringManager.get("E0050");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	/**
	 * 指定されたプロセス定義に属するプロセスインスタンスを
	 * フィルターで絞り込み、停止を行う。
	 *
	 * @param session セッションID
	 * @param pdid プロセス定義ID
	 * @param filter 検索用フィルター
	 * @throws InvalidSessionException 指定されたセッションが無効の場合
	 * @throws InvalidProcessDefinitionException 指定されたプロセス定義IDが有効でない場合
	 * @throws InvalidFilterException 指定されたフィルターが有効でない場合
	 */
	@Override
	public void terminateProcesses(String session, String pdid, Filter[] filter) throws InvalidSessionException, InvalidProcessDefinitionException, InvalidFilterException {

		String I4001 = StringManager.get("I4001")
			+ "(SessionID=" + session 
			+ ", ProcessDefinitionID=" + pdid 
			+ ", Filter=" + filter + ")";
		log.info(I4001);
		
		try {
			WorkflowServiceAccessor accessor = SingletonServices.getInstance().getAccessor(session);
			jp.co.argo21.nautica.workflow.wfmc.WorkflowAdminHandler wah = accessor.getWorkflowAdminHandler();
			wah.terminateProcesses(
					session, 
					pdid, 
					SoapMessageConverter.convertToGroupFilter(filter));
		} catch (InvalidSessionException e) {
			String errMsg = StringManager.get("E1015");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidProcessDefinitionException e) {
			String errMsg = StringManager.get("E0094");
			log.error(errMsg, e);
			throw e;
		} catch (InvalidFilterException e) {
			String errMsg = StringManager.get("E0004");
			log.error(errMsg, e);
			throw e;
		}
		
		String I4002 = StringManager.get("I4002");
	   log.info(I4002);
		
	}

	public WebServiceContext getContext() {
		return context;
	}
	
}